@@ -17,8 +17,6 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 #
 # HabitJewel: Track your habits
-# Version 0.1
-#
 
 VERSION = '0.1'
 
@@ -35,44 +33,72 @@ import sys
 import time
 
 from portrait import FremantleRotation
+from CellRendererClickablePixbuf import CellRendererClickablePixbuf
 
 import habitjewel_utils
 
 
-osso_c = osso.Context("org.maemo.habitjewel", VERSION, False)
+# Constants
+
+# Layout pseudo-constants
+UI_NORMAL = gtk.HILDON_UI_MODE_NORMAL
+UI_EDIT = gtk.HILDON_UI_MODE_EDIT
+BTN_ARR_HORIZ = hildon.BUTTON_ARRANGEMENT_HORIZONTAL
+BTN_ARR_VERT  = hildon.BUTTON_ARRANGEMENT_VERTICAL
+BTN_SIZE_FINGER = gtk.HILDON_SIZE_FINGER_HEIGHT
+BTN_SIZE_THUMB  = gtk.HILDON_SIZE_THUMB_HEIGHT
+WRAP_WIDTH_LANDSCAPE = 700
+WRAP_WIDTH_PORTRAIT = 380
+
+# Unused?
+WIN_PROG_IND = hildon.hildon_gtk_window_set_progress_indicator
+OSSO_CONTEXT = osso.Context("org.maemo.habitjewel", VERSION, False)
 
 
-fhsize = gtk.HILDON_SIZE_FINGER_HEIGHT
-horbtn = hildon.BUTTON_ARRANGEMENT_HORIZONTAL
-verbtn = hildon.BUTTON_ARRANGEMENT_VERTICAL
-ui_normal = gtk.HILDON_UI_MODE_NORMAL
-ui_edit = gtk.HILDON_UI_MODE_EDIT
-winprogind = hildon.hildon_gtk_window_set_progress_indicator
-thsize = gtk.HILDON_SIZE_THUMB_HEIGHT
+# Program constants
+TV_HABIT_LIST_ID           = 0
+TV_HABIT_LIST_DESC         = 1
+TV_HABIT_LIST_PCT_COMPLETE = 2
+TV_HABIT_LIST_PIXBUF       = 3
+TV_HABIT_LIST_INTVL_TYPE   = 4
+STATUS_FULFILLED_PCT    = 100
+STATUS_UNFULFILLED_PCT  = 0
+STATUS_UNKNOWN_PCT      = -1
+PIXBUF_FILE_FULFILLED   = "checkbox_checked.png"
+PIXBUF_FILE_UNFULFILLED = "checkbox_crossed.png"
+PIXBUF_FILE_UNKNOWN     = "checkbox_unchecked.png"
+
 
 # Initialisation
-HOME = os.path.expanduser("~")
-configdir = HOME + '/.habitjewel/'
-dbfile = configdir + 'database'
-logfile = configdir + 'log.txt'
+home = os.path.expanduser("~")
+config_dir = home + '/.habitjewel/'
+db_file = config_dir + 'database'
+log_file = config_dir + 'log.txt'
 
-#detect if running locally or not
-runningpath = sys.path[0]
 
-if '/opt' in runningpath:
+# Detect if running locally or not
+running_path = sys.path[0]
+if running_path.startswith('/opt'):
     locally = False
 else:
     locally = True
 
+if locally:
+    img_dir = 'pixmaps/'
+else:
+    app_dir = '/opt/habitjewel/'
+    img_dir = app_dir + 'pixmaps/'
+
+
 
 # Check for config dir, database. Create if necessary
-if not os.path.exists(configdir):
-    os.mkdir(configdir)
+if not os.path.exists(config_dir):
+    os.mkdir(config_dir)
 
-if os.path.exists(dbfile):
-    conn = sqlite3.connect(dbfile)
+if os.path.exists(db_file):
+    conn = sqlite3.connect(db_file)
 else:
-    conn = sqlite3.connect(dbfile)
+    conn = sqlite3.connect(db_file)
     cursor = conn.cursor()
     print 'creating new database'
     cursor.execute(
@@ -162,50 +188,67 @@ else:
     cursor.close()
 
 
-if locally:
-    imgdir = 'pixmaps/'
-else:
-    appdir = '/opt/habitjewel/'
-    imgdir = appdir + 'pixmaps/'
-
-
-# Get today's date
-view_date = datetime.date.today()
+# Get today's date and use that as the date displayed on startup
+today = datetime.date.today()
+view_date = today
 
 
 class MainWindow:
 
     def __init__(self):
-
         gettext.install('habitjewel','/opt/habitjewel/share/locale')
         self.program = hildon.Program()
         self.program.__init__()
         gtk.set_application_name("Habitjewel")
 
-        self.window = hildon.StackableWindow()
-        self.window.set_title(_("HabitJewel"))
-        self.window.set_default_size(800, 480)
-        self.window.connect("destroy", gtk.main_quit)
-        self.program.add_window(self.window)
+        self.top_window = hildon.StackableWindow()
+        self.top_window.set_title(_("HabitJewel"))
 
-        self.rotation = FremantleRotation('HabitJewel', None, VERSION, 0)
-        self.initialize_vars()
+        # N900-specific
+        self.osso_app_name = 'habitjewel'
+
+        self.rotation_obj = self.init_autorotation()
+
+        self.top_window.connect("destroy", gtk.main_quit)
+        self.top_window.get_screen().connect("size-changed", self.orientation_changed)
+        self.program.add_window(self.top_window)
+
+        #self.rotation = FremantleRotation('HabitJewel', None, VERSION, 0)
+        self.init_disp_orientation()
 
         self.fontsize = 15
 
         menu = self.make_menu()
-        self.window.set_app_menu(menu)
+        self.top_window.set_app_menu(menu)
 
-        vbox = self.home_screen()
-        self.window.add(vbox)
+        self.container = self.home_screen()
+        self.top_window.add(self.container)
 
-        self.window.show_all()
+        self.top_window.show_all()
 
 
-    def initialize_vars(self):
-        self.title = ''
-        self.mode = 'new'
-        self.cat = _('Daily')
+    def init_autorotation(self):
+        try:
+            import n900_maemo5_portrait
+
+            last_mode_number = 0 # Force auto-rotation
+            r_object = n900_maemo5_portrait.FremantleRotation(self.osso_app_name, \
+                main_window=self.top_window, mode=last_mode_number)
+            return r_object
+
+        except Exception:
+            print "LOGME: Initialising rotation object failed"
+
+
+    def init_disp_orientation(self):
+        global portrait
+        portrait = self.is_portrait()
+        if (portrait):
+            self.line_wrap_width = WRAP_WIDTH_PORTRAIT
+            self.button_size = BTN_SIZE_THUMB
+        else:
+            self.line_wrap_width = WRAP_WIDTH_LANDSCAPE
+            self.button_size = BTN_SIZE_FINGER
 
 
     def make_menu(self):
@@ -226,7 +269,6 @@ class MainWindow:
         button.connect("clicked", self.remove_habits)
         menu.append(button)
 
-
         button = gtk.Button(_("About"))
         button.connect("clicked", self.about)
         menu.append(button)
@@ -247,83 +289,127 @@ class MainWindow:
         return
 
     def home_screen(self):
-        vbox = gtk.VBox()
+        self.vbox_outer = gtk.VBox(False)
+        self.pan_area = hildon.PannableArea()
 
-        parea = hildon.PannableArea()
+        self.habit_list_tv = hildon.GtkTreeView(UI_NORMAL)
+        self.areaview = self.habit_list_tv.get_action_area_box()
 
-        self.habitlist_tv = hildon.GtkTreeView(ui_normal)
+        # HBox for 'prev' button
+        self.hbox_prev = gtk.HBox()
+        self.img_prev = gtk.image_new_from_icon_name("general_back", gtk.ICON_SIZE_SMALL_TOOLBAR)
+        self.hbox_prev.pack_start(self.img_prev)
+        # 'Prev' button
+        self.button_prev = hildon.Button(self.button_size, BTN_ARR_HORIZ)
+        self.button_prev.connect("clicked", self.prev_day)
+        self.button_prev.add(self.hbox_prev)
 
-        areaview = self.habitlist_tv.get_action_area_box()
-        self.habitlist_tv.set_action_area_visible(True)
+        # HBox for date display
+        self.hbox_date = gtk.HBox()
+        self.img_date = gtk.image_new_from_icon_name("general_calendar", gtk.ICON_SIZE_SMALL_TOOLBAR)
+        self.hbox_date.pack_start(self.img_date)
+        label_text = self.get_date_label_text(self)
+        self.date_label = gtk.Label(label_text)
+        self.hbox_date.pack_start(self.date_label)
 
-        hbox = gtk.HBox()
-        img = gtk.image_new_from_icon_name("general_back", gtk.ICON_SIZE_SMALL_TOOLBAR)
-        hbox.pack_start(img, True, True, 0)
+        # HBox for 'next' button
+        self.hbox_next = gtk.HBox()
+        self.img_next = gtk.image_new_from_icon_name("general_forward", gtk.ICON_SIZE_SMALL_TOOLBAR)
+        self.hbox_next.pack_start(self.img_next)
+        # 'Next' button
+        self.button_next = hildon.Button(self.button_size, BTN_ARR_HORIZ)
+        self.button_next.connect("clicked", self.next_day)
+        self.button_next.add(self.hbox_next)
 
-        button = hildon.Button(fhsize, horbtn)
-        button.connect("clicked", self.prev_day)
-        button.add(hbox)
-        areaview.pack_start(button, True, True, 0)
+        self.vbox_nav = gtk.VBox(False)
+        self.hbox_nav = gtk.HBox()
+        if (not self.is_portrait()):
+            self.hbox_nav.pack_start(self.button_prev)
+            self.hbox_nav.pack_start(self.hbox_date)
+            self.hbox_nav.pack_start(self.button_next)
+        else:
+            self.vbox_nav.pack_start(self.hbox_date, False, False, 5)
+            self.hbox_nav.pack_start(self.button_prev)
+            self.hbox_nav.pack_start(self.button_next)
 
-        hbox = gtk.HBox()
-        img = gtk.image_new_from_icon_name("general_calendar", gtk.ICON_SIZE_SMALL_TOOLBAR)
-        hbox.pack_start(img, True, True, 0)
-        self.date_label = gtk.Label()
-        self.draw_date_label(self)
-        hbox.pack_start(self.date_label, True, True, 0)
-        areaview.pack_start(hbox, True, True, 0)
-
-        hbox = gtk.HBox()
-        img = gtk.image_new_from_icon_name("general_forward", gtk.ICON_SIZE_SMALL_TOOLBAR)
-        hbox.pack_start(img, True, True, 0)
-
-        button = hildon.Button(fhsize, horbtn)
-        button.connect("clicked", self.next_day)
-        button.add(hbox)
-        areaview.pack_start(button, True, True, 0)
+        self.vbox_nav.pack_start(self.hbox_nav, False)
 
         self.habit_list_model = self.create_habit_list_model(self)
-        self.habitlist_tv.set_model(self.habit_list_model)
+        self.habit_list_tv.set_model(self.habit_list_model)
         self.prepare_habit_list(self)
 
-        parea.add(self.habitlist_tv)
-        vbox.pack_start(parea, True, True, 0)
+        self.pan_area.add(self.habit_list_tv)
 
-        return vbox
+        self.vbox_outer.pack_start(self.vbox_nav, False)
+        self.vbox_outer.pack_start(self.pan_area, True, True)
+
+        return self.vbox_outer
 
 
     def create_habit_list_model(self, widget):
-        lstore = gtk.ListStore(gobject.TYPE_INT, gobject.TYPE_STRING, gobject.TYPE_INT, \
-            gobject.TYPE_INT, gobject.TYPE_STRING)
-
-        #habitlist=[[id, 'title', 'unit', 'plural', 'target', 'goal', 'when', 
-        #   'interval_type', 'interval', 
-        #   'points', 'pct_complete', 'score', 'priority', 'cat_id', 'cat_title']...]
-
+        lstore = gtk.ListStore(int, str, int, gtk.gdk.Pixbuf, str)
         # add columns to the tree view
-        self.add_columns_to_habit_list(self.habitlist_tv)
+        self.add_columns_to_habit_list(self.habit_list_tv)
 
         return lstore
 
 
     def prepare_habit_list(self, widget):
-        habitlist = habitjewel_utils.get_habit_list(conn, view_date)
+        habit_list = habitjewel_utils.get_habit_list(conn, view_date)
 
-        for item in habitlist:
+        for item in habit_list:
             lstore_iter = self.habit_list_model.append()
-            if (item[10] == 100):
-                completed = 1
-            else:
-                completed = 0
-            self.habit_list_model.set(lstore_iter, 0, item[0], \
-                1, '<b>' + item[1] + '</b> ' + str(item[5]) \
-                + ' <i>' + item[6] + '</i>  for ' + str(item[9]) + ' pts', \
-                2, completed, 3, item[10], 4, item[7])
+            icon_pixbuf = self.get_pixbuf_filename_for_status (item['pct_complete'])
+ 
+            self.habit_list_model.set(lstore_iter, \
+                TV_HABIT_LIST_ID, \
+                    item['id'], \
+                TV_HABIT_LIST_DESC, \
+                    '<b>' + item['title'] + '</b> ' + str(item['goal']) \
+                    + ' <i>' + item['by_when'] + '</i>  for ' \
+                    + str(item['points']) + ' pts', \
+                TV_HABIT_LIST_PCT_COMPLETE, \
+                    item['pct_complete'], \
+                TV_HABIT_LIST_PIXBUF, \
+                    icon_pixbuf, \
+                TV_HABIT_LIST_INTVL_TYPE, \
+                    item['interval_type'] \
+            )
 
 
+    def get_pixbuf_filename_for_status(self, status):
+        if (status == STATUS_FULFILLED_PCT):
+            icon_filename = PIXBUF_FILE_FULFILLED
+        elif (status == STATUS_UNFULFILLED_PCT):
+            icon_filename = PIXBUF_FILE_UNFULFILLED
+        else:
+            icon_filename = PIXBUF_FILE_UNKNOWN
+
+        return gtk.gdk.pixbuf_new_from_file(img_dir + icon_filename)
+ 
+
     def add_columns_to_habit_list(self, treeview):
         # column for ID
-        column = gtk.TreeViewColumn('ID', gtk.CellRendererText(), text=0)
+        column = gtk.TreeViewColumn('ID', gtk.CellRendererText(), text=TV_HABIT_LIST_ID)
+        column.set_visible(False)
+        treeview.append_column(column)
+
+        # column for title
+        renderer = gtk.CellRendererText()
+        renderer.set_property('wrap-mode', gtk.WRAP_WORD)
+        renderer.set_property('wrap-width', self.line_wrap_width)
+        column = gtk.TreeViewColumn('Habit title', renderer, markup=TV_HABIT_LIST_DESC)
+        column.set_property("expand", True)
+        treeview.append_column(column)
+
+        # column for checkbox
+        checkbox = CellRendererClickablePixbuf()
+        checkbox.connect("clicked", self.habit_toggled, treeview)
+        column = gtk.TreeViewColumn('Status', checkbox, pixbuf=TV_HABIT_LIST_PIXBUF)
+        treeview.append_column(column)
+
+        # column for percent complete
+        column = gtk.TreeViewColumn('Percent complete', gtk.CellRendererText(), text=TV_HABIT_LIST_PCT_COMPLETE)
         column.set_visible(False)
         treeview.append_column(column)
 
@@ -332,46 +418,33 @@ class MainWindow:
         #column.set_visible(False)
         #treeview.append_column(column)
 
-        # column for title
-        renderer = gtk.CellRendererText()
-        renderer.set_property('wrap-mode', gtk.WRAP_WORD)
-        renderer.set_property('wrap-width', 700)
-        column = gtk.TreeViewColumn('Habit title', renderer, markup=1)
-        column.set_property("expand", True)
-        treeview.append_column(column)
-
-        # column for percent complete
-        column = gtk.TreeViewColumn('Percent complete', gtk.CellRendererText(), text=3)
-        column.set_visible(False)
-        treeview.append_column(column)
-
         # column for interval type
-        column = gtk.TreeViewColumn('Interval type', gtk.CellRendererText(), text=4)
-        column.set_visible(False)
-        treeview.append_column(column)
-
-        # column for checkbox
-        checkbox = gtk.CellRendererToggle()
-        checkbox.connect("toggled", self.habit_toggled, treeview)
-        column = gtk.TreeViewColumn('Status', checkbox, active=2)
-        column.set_property("expand", True)
-        treeview.append_column(column)
+        #column = gtk.TreeViewColumn('Interval type', gtk.CellRendererText(), text=4)
+        #column.set_visible(False)
+        #treeview.append_column(column)
 
 
     def habit_toggled(self, widget, row_num, treeview):
-        # Toggle habit completion status (0% / 100%)
+        # Toggle habit completion status (fulfilled / unfulfilled / unknown)
         model = treeview.get_model()
         iter = model.get_iter(row_num)
+        current_status_pct = model[iter][TV_HABIT_LIST_PCT_COMPLETE]
 
-        if (model[iter][3] < 100):
-            percent_complete = 100
+        if (current_status_pct == STATUS_FULFILLED_PCT):
+            percent_complete = STATUS_UNFULFILLED_PCT
+        elif (current_status_pct == STATUS_UNFULFILLED_PCT):
+            percent_complete = STATUS_UNKNOWN_PCT
         else:
-            percent_complete = 0
+            percent_complete = STATUS_FULFILLED_PCT
 
-        habitjewel_utils.set_percent_complete (conn, model[iter][0], model[iter][4], view_date, \
-            percent_complete)
-        model[iter][3] = percent_complete
-        model[iter][2] = not model[iter][2] 
+        habitjewel_utils.set_fulfillment_status (conn, \
+            model[iter][TV_HABIT_LIST_ID], \
+            model[iter][TV_HABIT_LIST_INTVL_TYPE], \
+            view_date, \
+            percent_complete \
+        )
+        model[iter][TV_HABIT_LIST_PIXBUF] = self.get_pixbuf_filename_for_status (percent_complete)
+        model[iter][TV_HABIT_LIST_PCT_COMPLETE] = percent_complete
 
 
     def prev_day(self, widget):
@@ -386,22 +459,29 @@ class MainWindow:
         self.redraw_habit_list(self)
 
 
-    def draw_date_label(self, widget):
+    def get_date_label_text(self, widget):
         date_disp = view_date.strftime("%a %d %B %Y")
-        self.date_label.set_text(date_disp)
+        return date_disp
 
 
     def redraw_habit_list(self, widget):
-        self.draw_date_label(self)
+        label_text = self.get_date_label_text(self)
+        self.date_label.set_text(label_text)
         self.habit_list_model.clear()
         self.prepare_habit_list(self)
+        checkbox_col = self.habit_list_tv.get_column(2)
+        if (view_date <= today):
+            checkbox_col.set_visible(True)
+        else:
+            checkbox_col.set_visible(False)
         self.redraw_window()
 
 
     def redraw_window(self):
-        self.window.queue_draw()
+        self.top_window.queue_draw()
 
 
+    #TODO: Everything below here
     def habit_edit_screen(self, widget, kind, habit_id):
         print "kind = "
         print kind
@@ -472,6 +552,7 @@ class MainWindow:
         else:
             return True
 
+
     #FIXME: there's a bug that occurs with these steps:
     # 1) Start application in landscape, Edit a habit
     # 2) Go to portrait, go to landscape
@@ -480,6 +561,21 @@ class MainWindow:
     # 5) Edit a habit in portrait
     # 6) Go to landscape
     def orientation_changed(self, screen):
+        global portrait
+        portrait = self.is_portrait()
+
+        # This works but may be possible to improve on
+        self.init_disp_orientation()
+        self.top_window.remove(self.container)
+
+        self.container = self.home_screen()
+        self.top_window.add(self.container)
+        self.top_window.show_all()
+
+        return
+
+        # Below example of reparenting etc from PyRecipe
+
         if not self.image.get_parent_window():
             print 'not in the right screen, doing nothing'
             return
